@@ -317,6 +317,7 @@ def collect_campaign_brief(payload: CollectCampaignBriefPayload) -> Dict[str, ob
         "Campaign planner is stateless; all files stay local when you run the `imgen` CLI.",
         "Use `imgen campaign templates list` for the full placement catalog after initialization.",
         "Run `imgen campaign init <campaign_id>` once the brief is ready, then add routes and placements.",
+        "Check progress with `imgen campaign status <campaign_id>` (or the FastMCP campaign_status_tool) whenever you resume work.",
         "Marketing work must stay on the campaign pipeline—do not fall back to slot tooling for missing variants.",
     ]
     if brief.generator == "mock" or DEFAULT_GENERATOR == "mock":
@@ -450,6 +451,21 @@ def plan_campaign_routes(payload: PlanCampaignRoutesPayload) -> Dict[str, object
         )
     )
 
+    status_cmd = [
+        "imgen",
+        "campaign",
+        "status",
+        payload.campaign_id,
+        "--json",
+    ]
+    actions.append(
+        _action(
+            step="status",
+            description="Summarize campaign state (useful for resume & QA)",
+            command=status_cmd,
+        )
+    )
+
     export_cmd = [
         "imgen",
         "campaign",
@@ -487,6 +503,7 @@ def plan_campaign_routes(payload: PlanCampaignRoutesPayload) -> Dict[str, object
     notes = [
         "Ensure the CLI is installed (pipx install imgen-cli) before executing the plan (imgen >= 0.1.18).",
         "Routes and placements are idempotent; rerunning the same commands updates existing files.",
+        "Use `imgen campaign status <id>` or the `campaign_status_tool` to inspect progress before/after generation and resume safely.",
         "Generation uses the summary-only flag so logs stay concise; drop it if you need detailed output.",
         "If a generate step fails, rerun the campaign command after fixing the issue—do not switch to slot-based `imgen gen` for marketing creatives.",
     ]
@@ -541,6 +558,14 @@ def plan_batch_generation(payload: PlanBatchGenerationPayload) -> Dict[str, obje
     if payload.summary_only:
         batch_cmd.append("--summary-only")
 
+    status_cmd = [
+        "imgen",
+        "campaign",
+        "status",
+        payload.campaign_id,
+        "--json",
+    ]
+
     export_cmd = [
         "imgen",
         "campaign",
@@ -565,6 +590,11 @@ def plan_batch_generation(payload: PlanBatchGenerationPayload) -> Dict[str, obje
             description="Run deterministic batch",
             command=batch_cmd,
         ),
+        _action(
+            step="status",
+            description="Summarize campaign state (useful for resume & QA)",
+            command=status_cmd,
+        ),
         _action(
             step="export",
             description=f"Export approved assets for {export_platform}",
@@ -575,6 +605,7 @@ def plan_batch_generation(payload: PlanBatchGenerationPayload) -> Dict[str, obje
     notes = [
         "Batch scaffold regenerates the spec each time; commit the file if you need CI reproducibility.",
         "Batch command streams JSONL logs into .imagemcp/campaigns/<id>/logs; attach them to debugging reports.",
+        "Use `imgen campaign status <id>` or the `campaign_status_tool` between runs to confirm outstanding variants before resuming.",
         "Keep campaign remediation inside the campaign CLI—never fall back to legacy slot commands for missing marketing assets.",
     ]
 
