@@ -15,7 +15,7 @@ from shutil import copy2
 from datetime import datetime
 from http.client import HTTPConnection
 from pathlib import Path
-from typing import Any, Dict, Iterable, Optional
+from typing import Any, Dict, Iterable, List, Optional, Sequence
 from urllib.parse import quote_plus
 
 import httpx
@@ -29,10 +29,18 @@ from . import (
     register_project,
     discover_project_root,
 )
-from .campaign import CampaignWorkspace, PlacementRef, CampaignRoute
+from .campaign import (
+    CampaignWorkspace,
+    PlacementRef,
+    CampaignRoute,
+    CampaignStatusReport,
+    PendingVariant,
+    build_campaign_status,
+)
 from .campaign.setup import (
     add_placement_to_campaign,
     add_route_from_args,
+    enrich_placement_geometry,
     ensure_campaign_exists,
     list_routes,
     load_route,
@@ -146,7 +154,10 @@ def build_parser() -> argparse.ArgumentParser:
     size_group.add_argument("--aspect-ratio", help="Aspect ratio (e.g. 16:9)")
     gen_parser.add_argument("--seed", type=int)
     gen_parser.add_argument("--provider", help="Provider id (e.g. openai)")
-    gen_parser.add_argument("--model", help="Model id (e.g. dall-e-3)")
+    gen_parser.add_argument(
+        "--model",
+        help="Model id (locked to google/gemini-2.5-flash-image-preview)",
+    )
     gen_parser.add_argument("--session-hint", help="Hint to include in session id")
     gen_parser.add_argument(
         "--generator",
@@ -382,6 +393,17 @@ def build_parser() -> argparse.ArgumentParser:
     campaign_placement_remove.add_argument("--json", action="store_true")
     campaign_placement_remove.set_defaults(func=cmd_campaign_placement_remove)
 
+    campaign_status_parser = campaign_sub.add_parser("status", help="Summarize campaign progress")
+    campaign_status_parser.add_argument("campaign_id", help="Campaign identifier")
+    campaign_status_parser.add_argument(
+        "--pending-limit",
+        type=int,
+        default=20,
+        help="Number of pending items to list when not using --json (default: 20)",
+    )
+    campaign_status_parser.add_argument("--json", action="store_true")
+    campaign_status_parser.set_defaults(func=cmd_campaign_status)
+
     campaign_generate_parser = campaign_sub.add_parser("generate", help="Generate variants for campaign placements")
     campaign_generate_parser.add_argument("campaign_id", help="Campaign identifier")
     campaign_generate_parser.add_argument(
@@ -400,6 +422,11 @@ def build_parser() -> argparse.ArgumentParser:
         action="store_true",
         help="Print a single-line summary when not using --json",
     )
+    campaign_generate_parser.add_argument(
+        "--restart-gallery",
+        action="store_true",
+        help="Force-restart the gallery server after generation",
+    )
     campaign_generate_parser.add_argument("--json", action="store_true")
     campaign_generate_parser.set_defaults(func=cmd_campaign_generate)
 
@@ -757,7 +784,20 @@ def cmd_generate(args: argparse.Namespace) -> int:
     aspect_ratio = args.aspect_ratio or payload.aspect_ratio
     seed = args.seed if args.seed is not None else payload.seed
     provider = args.provider or payload.provider or DEFAULT_PROVIDER
-    model = args.model or payload.model or default_model_for_provider(provider)
+    default_model = default_model_for_provider(provider)
+    requested_model = args.model or payload.model
+    manual_warnings: list[str] = []
+    if provider == "mock":
+        model = requested_model or default_model
+    else:
+        if requested_model and requested_model != default_model:
+            manual_warnings.append(
+                "Requested model '{requested}' is not supported; using '{locked}' instead.".format(
+                    requested=requested_model,
+                    locked=default_model,
+                )
+            )
+        model = default_model
     generator_name = (
         (args.generator if args.generator else None)
         or payload.generator
@@ -791,6 +831,9 @@ def cmd_generate(args: argparse.Namespace) -> int:
         }
         return _handle_generator_exception(exc, config, context=context, json_mode=args.json)
 
+    if manual_warnings:
+        generation.warnings[:0] = manual_warnings
+
     request = SessionRequest(request_text=request_text, created_by="cli")
     effective = EffectiveParameters(
         prompt=payload.prompt,
@@ -1176,7 +1219,10 @@ def cmd_campaign_init(args: argparse.Namespace) -> int:
     placement_refs: list[PlacementRef] = []
     if args.placements:
         placements = [item.strip() for item in args.placements.split(",") if item.strip()]
-        placement_refs = [PlacementRef(template_id=slug) for slug in placements]
+        placement_refs = [
+            enrich_placement_geometry(PlacementRef(template_id=slug))
+            for slug in placements
+        ]
 
     existing = workspace.config_path.exists()
     try:
@@ -1490,6 +1536,33 @@ def cmd_campaign_placement_remove(args: argparse.Namespace) -> int:
     return 0
 
 
+def cmd_campaign_status(args: argparse.Namespace) -> int:
+    try:
+        _, paths = _prepare_project(args, allow_create=False)
+    except (InvalidPathError, RuntimeError) as exc:
+        return _fail(str(exc))
+
+    workspace = CampaignWorkspace(paths, args.campaign_id)
+    try:
+        ensure_campaign_exists(workspace)
+    except FileNotFoundError as exc:
+        return _fail(str(exc))
+
+    try:
+        report = build_campaign_status(workspace)
+    except Exception as exc:  # pragma: no cover - defensive guard
+        return _fail(f"Failed to compute campaign status: {exc}")
+
+    if args.json:
+        print(json.dumps(report.to_dict(), indent=2))
+    else:
+        pending_limit = getattr(args, "pending_limit", 20) or 0
+        if pending_limit < 0:
+            pending_limit = 0
+        _print_campaign_status(report, pending_limit=pending_limit)
+    return 0
+
+
 def cmd_campaign_batch_scaffold(args: argparse.Namespace) -> int:
     try:
         _, paths = _prepare_project(args, allow_create=False)
@@ -1701,11 +1774,17 @@ def cmd_campaign_generate(args: argparse.Namespace) -> int:
         "generated_variants": stats.generated,
         "warnings": stats.warnings,
     }
+
+    force_restart_gallery = getattr(args, "restart_gallery", False) or GALLERY_ALWAYS_RESTART
+    _ensure_gallery_available(config, paths, GALLERY_HOST, GALLERY_PORT, force_restart_gallery)
+    gallery_url = _default_campaign_gallery_url(config.project_id, campaign_id)
+    result["gallery_url"] = gallery_url
+
     if args.json:
         print(json.dumps(result, indent=2))
     else:
         if getattr(args, "summary_only", False):
-            message = f"{campaign_id}: generated {stats.generated} variants"
+            message = f"{campaign_id}: generated {stats.generated} variants | gallery: {gallery_url}"
             if stats.warnings:
                 message += f" (warnings: {len(stats.warnings)})"
             print(message)
@@ -1715,6 +1794,7 @@ def cmd_campaign_generate(args: argparse.Namespace) -> int:
                 print("Warnings:")
                 for warning in stats.warnings:
                     print(f"  - {warning}")
+            print(f"View campaign in gallery: {gallery_url}")
     return 0
 
 
@@ -2066,6 +2146,12 @@ def _default_gallery_url(project_id: str, slot: str) -> str:
     return f"http://{GALLERY_HOST}:{GALLERY_PORT}/?project={project_param}&slot={slot_param}"
 
 
+def _default_campaign_gallery_url(project_id: str, campaign_id: str) -> str:
+    project_param = quote_plus(project_id)
+    campaign_param = quote_plus(campaign_id)
+    return f"http://{GALLERY_HOST}:{GALLERY_PORT}/?project={project_param}&campaign={campaign_param}"
+
+
 def _print_json(payload: Any) -> None:
     json.dump(payload, sys.stdout)
     sys.stdout.write("\n")
@@ -2106,6 +2192,140 @@ def _print_human_result(payload: Dict[str, Any]) -> None:
         print(payload)
 
 
+def _print_campaign_status(report: CampaignStatusReport, *, pending_limit: int = 20) -> None:
+    totals = report.totals
+    print(f"Campaign {report.campaign_id}")
+    print(
+        "Routes: {routes} | Placements: {placements} | Generated: {generated}/{expected} | Pending: {pending} | Extra: {extra}".format(
+            routes=totals.routes,
+            placements=totals.placements,
+            generated=totals.generated_variants,
+            expected=totals.expected_variants,
+            pending=totals.pending_variants,
+            extra=totals.extra_generated,
+        )
+    )
+
+    print()
+    print("Placements:")
+    if not report.placements:
+        print("  (no placements found)")
+    else:
+        for placement in report.placements:
+            template_note = ""
+            if placement.template_id and placement.template_id != placement.placement_id:
+                template_note = f" (template {placement.template_id})"
+            print(
+                "  - {placement}{template}: {generated}/{expected} generated, pending {pending}, extra {extra}".format(
+                    placement=placement.placement_id,
+                    template=template_note,
+                    generated=placement.generated_variants,
+                    expected=placement.expected_variants,
+                    pending=placement.pending_variants,
+                    extra=placement.extra_generated,
+                )
+            )
+            manifest_note = "present" if placement.manifest_present else "missing"
+            if placement.manifest_path:
+                print(f"    manifest: {manifest_note} ({placement.manifest_path})")
+            else:
+                print(f"    manifest: {manifest_note}")
+            total_routes = totals.routes or max(placement.routes_with_output, len(placement.routes_missing))
+            print(
+                "    routes with output: {done}/{total}".format(
+                    done=placement.routes_with_output,
+                    total=total_routes,
+                )
+            )
+            if placement.routes_missing:
+                preview = ", ".join(placement.routes_missing[:5])
+                if len(placement.routes_missing) > 5:
+                    preview += ", ..."
+                print(f"    routes pending: {preview}")
+
+    if report.routes:
+        print()
+        print("Routes:")
+        for route in report.routes:
+            summary = "  - {route}: {generated}/{expected} generated".format(
+                route=route.route_id,
+                generated=route.total_generated,
+                expected=route.total_expected,
+            )
+            if route.total_pending:
+                summary += f" (pending {route.total_pending})"
+            print(summary)
+            for placement in route.placements:
+                if (
+                    placement.expected_variants == 0
+                    and placement.generated_variants == 0
+                    and not placement.pending_variants
+                    and not placement.extra_generated
+                ):
+                    continue
+                detail = "    - {placement}: {generated}/{expected}".format(
+                    placement=placement.placement_id,
+                    generated=placement.generated_variants,
+                    expected=placement.expected_variants,
+                )
+                if placement.pending_variants:
+                    detail += " pending " + _format_variant_list(placement.pending_variants)
+                if placement.extra_generated:
+                    detail += " | extra " + _format_variant_list(placement.extra_generated)
+                print(detail)
+
+    if report.pending and pending_limit != 0:
+        print()
+        print("Next pending variants:")
+        for entry in _summarize_pending(report.pending, limit=pending_limit):
+            print(f"  - {entry}")
+
+    if report.orphan_files:
+        print()
+        print("Orphan files:")
+        for path in report.orphan_files:
+            print(f"  - {path}")
+
+    if report.missing_manifests:
+        print()
+        print("Placements missing manifests:")
+        for path in report.missing_manifests:
+            print(f"  - {path}")
+
+
+def _format_variant_index(index: int) -> str:
+    return f"v{index + 1:03d}"
+
+
+def _format_variant_list(indices: Sequence[int], limit: int = 3) -> str:
+    if not indices:
+        return ""
+    tokens = [_format_variant_index(idx) for idx in indices[:limit]]
+    if len(indices) > limit:
+        tokens.append("...")
+    return ", ".join(tokens)
+
+
+def _summarize_pending(pending: Sequence[PendingVariant], *, limit: int) -> List[str]:
+    if limit <= 0:
+        return []
+    ordered = sorted(pending, key=lambda item: (item.placement_id, item.route_id, item.variant_index))
+    lines: List[str] = []
+    max_items = min(limit, len(ordered))
+    for item in ordered[:max_items]:
+        lines.append(
+            "{placement} / {route} -> {variant}".format(
+                placement=item.placement_id,
+                route=item.route_id,
+                variant=_format_variant_index(item.variant_index),
+            )
+        )
+    remaining = len(ordered) - len(lines)
+    if remaining > 0:
+        lines.append(f"... ({remaining} more)")
+    return lines
+
+
 def _fail(message: str) -> int:
     print(message, file=sys.stderr)
     return 1
@@ -2187,6 +2407,24 @@ def _render_provider_error(exc: ProviderExecutionError) -> str:
     if model_id:
         lines.append(f"Model: {model_id}")
 
+    attempts_count = context.get("attempts")
+    text_only_attempts = context.get("text_only_attempts")
+    max_text_only_retries = context.get("max_text_only_retries")
+    if attempts_count or text_only_attempts or max_text_only_retries:
+        retry_bits: list[str] = []
+        if attempts_count:
+            retry_bits.append(f"attempts={attempts_count}")
+        if text_only_attempts:
+            retry_bits.append(f"text-only responses={text_only_attempts}")
+        if max_text_only_retries is not None:
+            retry_bits.append(f"auto-retry limit={max_text_only_retries}")
+        if retry_bits:
+            lines.append("Retry summary: " + ", ".join(retry_bits))
+
+    placeholder_token = context.get("placeholder_token")
+    if placeholder_token:
+        lines.append(f"Placeholder detected in prompt: '{placeholder_token}'")
+
     requested_total = context.get("requested_total")
     if requested_total:
         lines.append(f"Requested variants: {requested_total}")
@@ -2212,6 +2450,41 @@ def _render_provider_error(exc: ProviderExecutionError) -> str:
         lines.append("Response snippet:")
         lines.append(_indent_block(exc.body))
 
+    response_text_attempts = context.get("response_text_attempts")
+    if isinstance(response_text_attempts, list) and response_text_attempts:
+        lines.append("Provider text attempt breakdown:")
+        for attempt in response_text_attempts:
+            if not isinstance(attempt, dict):
+                lines.append(f"  - {_clip_text(str(attempt), 200)}")
+                continue
+            attempt_id = attempt.get("attempt")
+            attempt_reason = attempt.get("reason")
+            attempt_detail = attempt.get("detail")
+            line_bits = []
+            if attempt_id is not None:
+                line_bits.append(f"attempt {attempt_id}")
+            if attempt_reason:
+                line_bits.append(f"reason={attempt_reason}")
+            if attempt_detail:
+                line_bits.append(f"detail={_clip_text(str(attempt_detail), 200)}")
+            header_line = "  - " + ", ".join(line_bits) if line_bits else "  -"
+            lines.append(header_line)
+            attempt_text = attempt.get("text")
+            if isinstance(attempt_text, list):
+                for idx, snippet in enumerate(attempt_text, start=1):
+                    lines.append(f"      [{idx}] {_clip_text(str(snippet), 200)}")
+
+    response_text = context.get("response_text")
+    if isinstance(response_text, list) and response_text:
+        lines.append("Provider text replies (flattened):")
+        for idx, snippet in enumerate(response_text, start=1):
+            lines.append(f"  [{idx}] {_clip_text(str(snippet), 200)}")
+
+    if response_text_attempts or response_text:
+        lines.append(
+            "Action: confirm the prompt contains concrete visual directions (no TODO placeholders) before retrying."
+        )
+
     consumed = {
         "campaign_id",
         "slot",
@@ -2225,6 +2498,12 @@ def _render_provider_error(exc: ProviderExecutionError) -> str:
         "aspect_ratio",
         "request_id",
         "retry_after",
+        "response_text",
+        "response_text_attempts",
+        "attempts",
+        "text_only_attempts",
+        "max_text_only_retries",
+        "placeholder_token",
     }
     extras = {
         key: value
