@@ -22,6 +22,27 @@ except AttributeError:  # pragma: no cover - older Pillow fallback
 
 _DEFAULT_SIZE = (1024, 1024)
 
+_PLACEHOLDER_PATTERNS = (
+    "todo: fill prompt",
+    "todo: fill summary",
+    "todo: fill campaign objective",
+    "todo: update campaign objective",
+)
+
+_MAX_TEXT_ONLY_RETRIES = 3
+
+
+def _detect_placeholder_prompt(prompt: str) -> Optional[str]:
+    normalized = (prompt or "").strip().lower()
+    if not normalized:
+        return None
+    for pattern in _PLACEHOLDER_PATTERNS:
+        if pattern in normalized:
+            return pattern
+    if "todo" in normalized:
+        return "todo"
+    return None
+
 
 class ProviderExecutionError(RuntimeError):
     """Raised when an upstream provider cannot complete an image request."""
@@ -234,18 +255,46 @@ class OpenRouterGenerator:
             "aspect_ratio": aspect_ratio,
         }
 
+        placeholder_token = _detect_placeholder_prompt(prompt)
+        if placeholder_token:
+            context = {
+                **request_context,
+                "reason": "placeholder_prompt",
+                "placeholder_token": placeholder_token,
+            }
+            raise ProviderExecutionError(
+                provider=provider_name,
+                generator="openrouter",
+                message=(
+                    "Prompt still contains placeholder text (found '{token}'). "
+                    "Replace the TODO copy with concrete image instructions before generating."
+                ).format(token=placeholder_token),
+                context=context,
+            )
+
+        max_text_retries = _MAX_TEXT_ONLY_RETRIES
+        attempts = 0
+        text_only_streak = 0
+        aggregated_text_snippets: List[str] = []
+        text_attempt_records: List[Dict[str, object]] = []
+        last_failure_message: Optional[str] = None
+        last_failure_reason = "empty_response"
+
         try:
             while len(images) < total:
+                attempts += 1
                 remaining = total - len(images)
                 batch_size = min(remaining, self.max_workers)
                 payloads = [
                     self._build_payload(prompt, model_id, provider_config, reference_image)
                     for _ in range(batch_size)
                 ]
-                responses = self._execute_batch(payloads)
-                if not responses:
-                    break
+                responses = self._execute_batch(payloads) or []
                 new_images = False
+                attempt_snippets: List[str] = []
+                attempt_reason: Optional[str] = None
+                attempt_detail: Optional[str] = None
+
                 for response in responses:
                     if response is None:
                         continue
@@ -258,26 +307,69 @@ class OpenRouterGenerator:
                     if resp_images:
                         images.extend(resp_images)
                         new_images = True
-                    if len(images) >= total:
-                        break
-                if not new_images:
+                        if len(images) >= total:
+                            break
+                        continue
+
+                    reason, detail, text_snippets = _diagnose_no_image_response(response)
+                    attempt_reason = attempt_reason or reason
+                    if detail and not attempt_detail:
+                        attempt_detail = detail
+                    if text_snippets:
+                        attempt_snippets.extend(text_snippets)
+
+                if new_images:
+                    text_only_streak = 0
+                    continue
+
+                text_only_streak += 1
+                if attempt_reason:
+                    last_failure_reason = attempt_reason
+                if attempt_detail:
+                    last_failure_message = attempt_detail
+                if attempt_snippets:
+                    aggregated_text_snippets.extend(attempt_snippets)
+
+                record: Dict[str, object] = {
+                    "attempt": attempts,
+                    "reason": attempt_reason or "empty_response",
+                }
+                if attempt_detail:
+                    record["detail"] = attempt_detail
+                if attempt_snippets:
+                    record["text"] = attempt_snippets
+                text_attempt_records.append(record)
+
+                if text_only_streak > max_text_retries:
                     break
 
             if not images:
                 debug_message = ""
-                reason = "empty_response"
-                failure_message: Optional[str] = None
-                text_snippets: List[str] = []
                 if last_response is not None:
                     try:
                         debug_message = json.dumps(last_response, indent=2)
                     except Exception:
                         debug_message = str(last_response)
-                    reason, failure_message, text_snippets = _diagnose_no_image_response(last_response)
-                message_text = failure_message or "OpenRouter did not return any image files."
-                context = {**request_context, "reason": reason}
-                if text_snippets:
-                    context["response_text"] = text_snippets
+
+                failure_message = last_failure_message
+                if failure_message:
+                    message_text = failure_message
+                else:
+                    message_text = "OpenRouter did not return any image files."
+                if attempts:
+                    message_text += " Ensure the prompt is an image-generation request and try again."
+
+                context = {
+                    **request_context,
+                    "reason": last_failure_reason,
+                    "attempts": attempts,
+                    "text_only_attempts": text_only_streak,
+                    "max_text_only_retries": max_text_retries,
+                }
+                if aggregated_text_snippets:
+                    context["response_text"] = aggregated_text_snippets
+                if text_attempt_records:
+                    context["response_text_attempts"] = text_attempt_records
                 raise ProviderExecutionError(
                     provider=provider_name,
                     generator="openrouter",
