@@ -132,6 +132,9 @@ def collect_context_questions(payload: CollectContextPayload) -> Dict[str, objec
         "Gather either an explicit size (e.g. 1024x1024) or an aspect ratio (e.g. 16:9); the planner will reject requests without geometry so the CLI receives clear instructions.",
         "The CLI promotes variant #0 immediately so previews update right away; other picks are available via the gallery server.",
         "Seeds are optional and provider-specific. Include one for reproducibility when the provider supports it.",
+        "Image generation is locked to Gemini 2.5 (`{model}`); leave provider/model at their defaults.".format(
+            model=default_model_for_provider(DEFAULT_PROVIDER)
+        ),
     ]
     notes.extend(_geometry_hint_notes(constraints))
     if known.projectRoot:
@@ -171,13 +174,29 @@ def plan_image_job(payload: PlanPayload) -> Dict[str, object]:
     else:
         provider = DEFAULT_PROVIDER
         provider_explicit = False
+    desired_model = default_model_for_provider(provider)
+    model_notes: List[str] = []
     raw_model = plan_input.constraints.get("model")
-    if raw_model:
-        model = str(raw_model)
-        model_explicit = True
+    requested_model = str(raw_model) if raw_model else None
+    if provider == "mock":
+        model = requested_model or desired_model
+        model_explicit = bool(requested_model)
     else:
-        model = default_model_for_provider(provider)
-        model_explicit = False
+        if requested_model and requested_model != desired_model:
+            model_notes.append(
+                "Requested model '{requested}' is not supported; locking to '{locked}' for stability.".format(
+                    requested=requested_model,
+                    locked=desired_model,
+                )
+            )
+        model = desired_model
+        model_explicit = True
+        model_notes.append(
+            "Gemini 2.5 ('{model}') is the only approved model; do not change provider/model in the CLI command.".format(
+                model=model,
+            )
+        )
+    plan_input.constraints["model"] = model
     raw_generator = plan_input.constraints.get("generator")
     if raw_generator:
         generator = str(raw_generator)
@@ -223,6 +242,8 @@ def plan_image_job(payload: PlanPayload) -> Dict[str, object]:
             "Available provider/model combinations are documented in resource `setup://imgen-parameters`.",
         ]
     )
+    if model_notes:
+        notes.extend(model_notes)
     if project_context.gallery_url:
         notes.append(
             "After running the CLI, open the gallery at {url} to review and share variants.".format(
