@@ -118,6 +118,91 @@ def test_cli_models_unknown_provider(capsys):
     assert "Unknown provider" in captured.err
 
 
+def test_cli_campaign_generate_emits_gallery_url(tmp_path, capsys, monkeypatch):
+    project_root = tmp_path
+    campaign_id = "camp_gallery"
+
+    exit_code = cli_main(
+        [
+            "--project-root",
+            str(project_root),
+            "campaign",
+            "init",
+            campaign_id,
+            "--objective",
+            "Gallery smoke",
+            "--placements",
+            "meta_feed_square",
+            "--json",
+        ]
+    )
+    assert exit_code == 0
+    capsys.readouterr()
+
+    exit_code = cli_main(
+        [
+            "--project-root",
+            str(project_root),
+            "campaign",
+            "route",
+            "add",
+            campaign_id,
+            "main_story",
+            "--summary",
+            "Main story",
+            "--prompt-template",
+            "Test prompt",
+            "--json",
+        ]
+    )
+    assert exit_code == 0
+    capsys.readouterr()
+
+    exit_code = cli_main(
+        [
+            "--project-root",
+            str(project_root),
+            "campaign",
+            "placement",
+            "add",
+            campaign_id,
+            "meta_feed_square",
+            "--variants",
+            "1",
+            "--json",
+        ]
+    )
+    assert exit_code == 0
+    capsys.readouterr()
+
+    gallery_calls = []
+
+    def fake_ensure_gallery_available(config, paths, host, port, force_restart):
+        gallery_calls.append((config.project_id, host, port, force_restart))
+
+    monkeypatch.setattr("imagemcp.cli._ensure_gallery_available", fake_ensure_gallery_available)
+
+    exit_code = cli_main(
+        [
+            "--project-root",
+            str(project_root),
+            "campaign",
+            "generate",
+            campaign_id,
+            "--generator",
+            "mock",
+            "--json",
+        ]
+    )
+    assert exit_code == 0
+    output = json.loads(capsys.readouterr().out)
+
+    assert "gallery_url" in output
+    assert output["gallery_url"].startswith("http://")
+    assert gallery_calls
+    assert gallery_calls[0][0]
+
+
 def test_cli_invalid_model_guidance(tmp_path, capsys, monkeypatch):
     def fake_build_generator(output_dir, kind):
         raise RuntimeError(
