@@ -7,6 +7,7 @@ from typing import Dict
 
 import pytest
 
+from imagemcp.defaults import default_model_for_provider
 from imagemcp.planner import (
     CollectContextPayload,
     KnownContext,
@@ -163,9 +164,29 @@ def test_plan_image_job_defaults_provider_and_model_with_aspect_ratio():
     assert "--aspect-ratio" in command
     assert "--provider" not in command
     assert "--project-root" in command
-    assert "--model" not in command
-    assert "model" not in result["cli"]["stdin"]
+    assert "--model" in command
+    assert result["cli"]["stdin"]["model"] == "google/gemini-2.5-flash-image-preview"
     assert "provider" not in result["cli"]["stdin"]
+    assert plan["constraints"]["model"] == "google/gemini-2.5-flash-image-preview"
+    assert any("Gemini 2.5" in note for note in result["notes"])
+
+
+def test_plan_image_job_overrides_custom_model_to_gemini(tmp_path):
+    constraints = PlanConstraints(width=512, height=512, model="other/unsupported-model")
+    payload = PlanPayload(
+        slot="hero",
+        requestText="Override model",
+        constraints=constraints,
+        projectRoot=str(tmp_path),
+    )
+    result = plan_image_job(payload)
+    expected_model = default_model_for_provider()
+    plan = result["plan"]
+    assert plan["model"] == expected_model
+    assert plan["constraints"]["model"] == expected_model
+    cli_stdin = result["cli"]["stdin"]
+    assert cli_stdin["model"] == expected_model
+    assert any("locking to" in note for note in result["notes"])
 
 
 def test_normalize_plan_payload_accepts_string():
